@@ -66,11 +66,7 @@ from_string(const std::string& s,
 
     try
     {
-        iss >> f >> ret;
-        if (iss.fail() || not iss.eof())
-        {
-            throw NotFound();
-        }
+        if ((iss >> f >> ret).fail()) throw NotFound();
     }
     catch (gu::Exception& e)
     {
@@ -97,11 +93,7 @@ from_string<void*>(const std::string& s,
     std::istringstream iss(s);
     void*              ret;
 
-    iss >> std::hex >> ret;
-    if (iss.fail() || not iss.eof())
-    {
-        throw NotFound();
-    }
+    if ((iss >> std::hex >> ret).fail()) throw NotFound();
 
     return ret;
 }
@@ -165,53 +157,24 @@ public:
     template <class T> void operator()(T* t) { delete t; }
 };
 
-typedef std::ios_base& (*base_t) (std::ios_base& str);
-
-template <base_t base = std::hex,
-          typename T  = unsigned long long,
-          bool prefix = false>
-class PrintBase
+/*! swap method for arrays, which does't seem to be built in all compilers */
+template <typename T, size_t N>
+inline void
+swap_array(T (&a)[N], T (&b)[N])
 {
-public:
-    explicit PrintBase(T t) : val_(t) {}
-    void print(std::ostream& os) const
-    {
-        using namespace std;
-
-        ios_base::fmtflags const old_flags(os.flags());
-        char               const old_fill (os.fill());
-
-        int width(sizeof(T) * 2); // default hex width
-        if (base == oct) width = width * 1.333334 + 0.5;
-
-        if (prefix) os << showbase;
-
-        os << internal << base << setfill('0') << setw(width) << val_;
-
-        os.flags(old_flags);
-        os.fill(old_fill);
-    }
-private:
-    T const val_;
-};
-
-template <base_t base, typename T>
-std::ostream& operator << (std::ostream& os, const PrintBase<base, T>& b)
-{
-    b.print(os); return os;
+    for (size_t n(0); n < N; ++n) std::swap(a[n], b[n]);
 }
 
 /*! template to do arithmetics on void and byte pointers, compiler will
  *  catch anything else.
  * @return input type */
-template <typename T, typename PtrOffsetType>
+template <typename T>
 inline T*
-ptr_offset(T* ptr, PtrOffsetType i) { return static_cast<byte_t*>(ptr) + i; }
+ptr_offset(T* ptr, ptrdiff_t i) { return static_cast<byte_t*>(ptr) + i; }
 
-template <typename T, typename PtrOffsetType>
+template <typename T>
 inline const T*
-ptr_offset(const T* ptr, PtrOffsetType i)
-    { return static_cast<const byte_t*>(ptr)+i; }
+ptr_offset(const T* ptr, ptrdiff_t i) { return static_cast<const byte_t*>(ptr)+i; }
 
 } // namespace gu
 
