@@ -169,7 +169,7 @@ group_redo_last_applied (gcs_group_t* group)
 {
     long       n;
     long       last_node    = -1;
-    gu_seqno_t last_applied = GU_LONG_LONG_MAX;
+    gu_seqno_t last_applied = GU_LLONG_MAX;
 
     for (n = 0; n < group->num; n++) {
         const gcs_node_t* const node = &group->nodes[n];
@@ -542,8 +542,9 @@ gcs_group_handle_uuid_msg  (gcs_group_t* group, const gcs_recv_msg_t* msg)
 
     if (GCS_GROUP_WAIT_STATE_UUID == group->state &&
         0 == msg->sender_idx /* check that it is from the representative */) {
-        group->state_uuid = *(gu_uuid_t*)msg->buf;
-        group->state      = GCS_GROUP_WAIT_STATE_MSG;
+//        group->state_uuid = *(gu_uuid_t*)msg->buf;
+        gu_uuid_copy(&group->state_uuid, (const gu_uuid_t*)msg->buf);
+        group->state = GCS_GROUP_WAIT_STATE_MSG;
     }
     else {
         gu_warn ("Stray state UUID msg: " GU_UUID_FORMAT
@@ -805,15 +806,23 @@ gcs_group_handle_sync_msg  (gcs_group_t* group, const gcs_recv_msg_t* msg)
         return (sender_idx == group->my_idx);
     }
     else {
-        if (GCS_NODE_STATE_SYNCED != sender->status) {
-            gu_warn ("SYNC message sender from non-JOINED %d.%d (%s). Ignored.",
-                     sender_idx, sender->segment, sender->name);
-        }
-        else {
+        if (GCS_NODE_STATE_SYNCED == sender->status) {
             gu_debug ("Redundant SYNC message from %d.%d (%s).",
                       sender_idx, sender->segment, sender->name);
         }
-        return 0;
+        else if (GCS_NODE_STATE_DONOR == sender->status) {
+            // this is possible with quick succession of desync()/resync() calls
+            gu_debug ("SYNC message from %d.%d (%s, DONOR). Ignored.",
+                      sender_idx, sender->segment, sender->name);
+        }
+        else {
+            gu_warn ("SYNC message from non-JOINED %d.%d (%s, %s). Ignored.",
+                     sender_idx, sender->segment, sender->name,
+                     gcs_node_state_to_str(sender->status));
+        }
+
+        /* signal sender that it didn't work */
+        return -ERESTART * (sender_idx == group->my_idx);
     }
 }
 
